@@ -1,14 +1,14 @@
 ;;; jinx.el --- Enchanted Spell Checker -*- lexical-binding: t -*-
 
-;; Copyright (C) 2023 Free Software Foundation, Inc.
+;; Copyright (C) 2023-2024 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2023
-;; Version: 1.1
+;; Version: 1.2
 ;; Package-Requires: ((emacs "27.1") (compat "29.1.4.4"))
 ;; Homepage: https://github.com/minad/jinx
-;; Keywords: convenience, wp
+;; Keywords: convenience, text
 
 ;; This file is part of GNU Emacs.
 
@@ -259,7 +259,8 @@ checking."
 ;;;; Internal variables
 
 (defvar jinx--reschedule-hooks
-  '(window-state-change-hook window-scroll-functions post-command-hook)
+  '(window-selection-change-functions window-scroll-functions
+    window-state-change-hook post-command-hook)
   "Hooks which reschedule the spell checking timer, see `jinx--reschedule'.")
 
 (defvar jinx--predicates
@@ -270,13 +271,16 @@ checking."
 Predicate should return t if the word before point is valid.
 Predicate may return a position to skip forward.")
 
-(defvar jinx--timer nil
+(defvar jinx--timer (timer-create)
   "Global timer to check pending regions.")
 
 (defvar jinx--base-syntax-table
   (let ((st (make-syntax-table)))
     (modify-syntax-entry ?$ "_" st)
     (modify-syntax-entry ?% "_" st)
+    (modify-syntax-entry '(#xe000 . #xf8ff) "_" st)    ;; Priv. use area
+    (modify-syntax-entry '(#xf0000 . #xffffd) "_" st)  ;; Supp. priv. use area A
+    (modify-syntax-entry '(#x100000 . #x10fffd) "_" st);; Supp. priv. use area B
     (modify-syntax-entry '(#x80 . #x9f) "_" st)        ;; Control characters
     (modify-syntax-entry '(#x2150 . #x2bff) "_" st)    ;; Number Forms - Misc. Arrows
     (modify-syntax-entry '(#xfe00 . #xfe0f) "_" st)    ;; Variation Selectors
@@ -539,7 +543,7 @@ If CHECK is non-nil, always check first."
 
 (defun jinx--timer-handler ()
   "Global timer handler, checking the pending regions in all windows."
-  (setq jinx--timer nil)
+  (timer-set-function jinx--timer nil)
   (dolist (frame (frame-list))
     (dolist (win (window-list frame 'no-miniwindow))
       (when-let ((buffer (window-buffer win))
@@ -549,19 +553,19 @@ If CHECK is non-nil, always check first."
 
 (defun jinx--reschedule (&rest _)
   "Restart the global idle timer."
-  (when jinx--timer
+  (when (timer--function jinx--timer)
     (cancel-timer jinx--timer)
-    (setq jinx--timer nil))
+    (timer-set-function jinx--timer nil))
   (jinx--schedule))
 
 (defun jinx--schedule ()
   "Start the global idle timer."
-  (when (and (not jinx--timer)
+  (when (and (not (timer--function jinx--timer))
              (not completion-in-region-mode) ;; Corfu completion
              (get-buffer-window)) ;; Buffer visible
-    (setq jinx--timer
-          (run-with-idle-timer jinx-delay
-                               nil #'jinx--timer-handler))))
+    (timer-set-function jinx--timer #'jinx--timer-handler)
+    (timer-set-idle-time jinx--timer jinx-delay)
+    (timer-activate-when-idle jinx--timer t)))
 
 (defun jinx--load-module ()
   "Compile and load dynamic module."
@@ -597,6 +601,11 @@ If CHECK is non-nil, always check first."
                   (pop-to-buffer (current-buffer))
                   (error msg)))))
           (setq mod-file (expand-file-name mod-name))))
+      ;; Initialize Mac spell checker to avoid dead lock (gh:minad/jinx#91).
+      (when (and (eq window-system 'mac) (fboundp 'mac-do-applescript))
+        (mac-do-applescript
+         "use framework \"AppKit\"
+          set spellChecker to current application's NSSpellChecker's sharedSpellChecker()"))
       (module-load mod-file))))
 
 (defmacro jinx--correct-guard (&rest body)
